@@ -9,13 +9,13 @@ module Facebook.Base
     , fbhttp
     , fbhttpHelper
     , httpCheck
+    , apiVersion
     ) where
 
 import Control.Applicative
 import Control.Monad (mzero)
 import Control.Monad.IO.Class (MonadIO)
 import Data.ByteString.Char8 (ByteString)
-import Data.Default (def)
 import Data.Text (Text)
 import Data.Typeable (Typeable)
 
@@ -24,31 +24,36 @@ import Control.Monad.Trans.Class (MonadTrans)
 import Control.Monad.Trans.Control (MonadBaseControl)
 import qualified Control.Monad.Trans.Resource as R
 import qualified Data.Aeson as A
-import qualified Data.Attoparsec.Char8 as AT
+import qualified Data.Attoparsec.ByteString.Char8 as AT
 import qualified Data.ByteString as B
 import qualified Data.Conduit as C
 import qualified Data.Conduit.Attoparsec as C
 import qualified Data.Conduit.List as CL
+import qualified Data.Conduit.Binary as CB
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as TE
 import qualified Network.HTTP.Conduit as H
 import qualified Network.HTTP.Types as HT
-
+import qualified Data.ByteString.Lazy as L
+#if !MIN_VERSION_http_client(0,4,30)
+import Data.Default (def)
+#endif
 #if DEBUG
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Text.Printf (printf)
-import qualified Data.ByteString.Lazy as L
 #endif
 
 
 import Facebook.Types
 import Facebook.Monad
 
+apiVersion :: Text
+apiVersion = "v2.8"
 
 -- | A plain 'H.Request' to a Facebook API.  Use this instead of
 -- 'def' when creating new 'H.Request'@s@ for Facebook.
 fbreq :: Monad m =>
-         Text                        -- ^ Path.
+         Text                        -- ^ Path. Should start from "/".
       -> Maybe (AccessToken anyKind) -- ^ Access token.
       -> HT.SimpleQuery              -- ^ Parameters.
       -> FacebookT anyAuth m H.Request
@@ -57,6 +62,22 @@ fbreq path mtoken query =
       let host = case tier of
                    Production -> "graph.facebook.com"
                    Beta ->  "graph.beta.facebook.com"
+#if MIN_VERSION_http_client(0,4,30)
+      in H.defaultRequest { H.secure        = True
+             , H.host          = host
+             , H.port          = 443
+             , H.path          = TE.encodeUtf8 ("/" <> apiVersion <> path)
+             , H.redirectCount = 3
+             , H.queryString   =
+                 HT.renderSimpleQuery False $
+                 maybe id tsq mtoken query
+#if MIN_VERSION_http_client(0,5,0)
+             , H.responseTimeout = H.responseTimeoutMicro 120000000 -- 2 minutes
+#else
+             , H.responseTimeout = Just 120000000 -- 2 minutes
+#endif
+             }
+#else
       in def { H.secure        = True
              , H.host          = host
              , H.port          = 443
@@ -67,6 +88,7 @@ fbreq path mtoken query =
                  maybe id tsq mtoken query
              , H.responseTimeout = Just 120000000 -- 2 minutes
              }
+#endif
 
 
 -- | Internal class for types that may be passed on queries to
@@ -153,21 +175,32 @@ fbhttpHelper :: (MonadBaseControl IO m, R.MonadResource m) =>
              -> H.Request
              -> m (H.Response (C.ResumableSource m ByteString))
 fbhttpHelper manager req = do
+#if MIN_VERSION_http_client(0,5,0)
+  let req' = req { H.checkResponse = \_ _ -> return () }
+#else
   let req' = req { H.checkStatus = \_ _ _ -> Nothing }
+#endif
 #if DEBUG
   _ <- liftIO $ printf "fbhttp doing request\n\tmethod: %s\n\tsecure: %s\n\thost: %s\n\tport: %s\n\tpath: %s\n\tqueryString: %s\n\trequestHeaders: %s\n" (show $ H.method req') (show $ H.secure req') (show $ H.host req') (show $ H.port req') (show $ H.path req') (show $ H.queryString req') (show $ H.requestHeaders req')
 #endif
   response <- H.http req' manager
   let status  = H.responseStatus    response
       headers = H.responseHeaders   response
-      cookies = H.responseCookieJar response
 #if DEBUG
   _ <- liftIO $ printf "fbhttp response status: %s\n" (show status)
 #endif
   if isOkay status
     then return response
     else do
+#if MIN_VERSION_http_client(0,5,0)
+      fullResp <- H.responseBody response C.$$+- CB.sinkLbs
+      let res' = fmap (const ()) response
+      let statusexc = H.HttpExceptionRequest req $ H.StatusCodeException res' (L.toStrict fullResp)
+#else
+      let cookies = H.responseCookieJar response
       let statusexc = H.StatusCodeException status headers cookies
+#endif
+
       val <- E.try $ asJsonHelper response
       case val :: Either E.SomeException FacebookException of
         Right fbexc -> E.throw fbexc
@@ -200,7 +233,12 @@ httpCheck :: (MonadBaseControl IO m, R.MonadResource m) =>
 httpCheck req = runResourceInFb $ do
   manager <- getManager
   let req' = req { H.method      = HT.methodHead
-                 , H.checkStatus = \_ _ _ -> Nothing }
+#if MIN_VERSION_http_client(0,5,0)
+     , H.checkResponse = \_ _ -> return ()
+#else
+     , H.checkStatus = \_ _ _ -> Nothing
+#endif
+  }
   isOkay . H.responseStatus <$> lift (H.httpLbs req' manager)
   -- Yes, we use httpLbs above so that we don't have to worry
   -- about consuming the responseBody.  Note that the
